/* Project name:
     Battery
 * Copyright:
     (c) Mikroelektronika, 2011.
 * Revision History:
     20110512(TL):
       - initial release;
 * Description:
     Simple battery charger demo
 * Test configuration:
     MCU:             dsPIC33FJ256GP710A
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70593C.pdf
     Dev.board:       mikroMMB for dsPIC33 hw.rev.1.05
                      http://www.mikroe.com/eng/products/view/586/mikrommb-for-dspic33-board/
     Oscillator:      HS-PLL, 32.0000 MHz Crystal
     Ext. Modules:    Li-Polymer Battery - ac:Battery
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/eng/products/view/231/mikroc-pro-for-dspic30-33-and-pic24/
 * NOTES:
    none.*/

#include <built_in.h>


// TFT module connections
char TFT_DataPort at LATE;
char TFT_DataPort_Direction at TRISE;
sbit TFT_RST at LATC1_bit;
sbit TFT_RS at LATB15_bit;
sbit TFT_CS at LATF12_bit;
sbit TFT_RD at LATD5_bit;
sbit TFT_WR at LATD4_bit;
sbit TFT_RST_Direction at TRISC1_bit;
sbit TFT_RS_Direction at TRISB15_bit;
sbit TFT_CS_Direction at TRISF12_bit;
sbit TFT_RD_Direction at TRISD5_bit;
sbit TFT_WR_Direction at TRISD4_bit;
sbit TFT_D7 at LATE7_bit;
sbit TFT_D6 at LATE6_bit;
sbit TFT_D5 at LATE5_bit;
sbit TFT_D4 at LATE4_bit;
sbit TFT_D3 at LATE3_bit;
sbit TFT_D2 at LATE2_bit;
sbit TFT_D1 at LATE1_bit;
sbit TFT_D0 at LATE0_bit;

sbit STAT at RC3_bit;
sbit STAT_direction at TRISC3_bit;

char i = 0, sample = 0;
unsigned int adc_rd;
char txt[14];
char txt1[1];
char j;
float voltage_value;


void main() {
  TRISB8_bit = 1;
  AD1PCFGL = 0xFEFF;
  AD1PCFGH = 0xFFFF;

  CM1CON.B15 = 0;
  CM2CON.B15 = 0;
  CM3CON.B15 = 0;
  Delay_ms(150);

  //ADC1_Init();
  ADC1_Init_Advanced(_ADC_EXTERNAL_REF);
  TFT_Init(320, 240);

  UART2_Init(19200);                // Initialize UART module at 19200 bps


  TFT_Init(320,240);                // Initialize TFT
  TFT_Fill_Screen(CL_WHITE);        // Clear screen                              // Clear TFT

  TFT_Write_Text("Battery   charging   example.", 50, 100);
  Delay_ms(2000);
  TFT_Fill_Screen(CL_WHITE);        // Fills screen memory block with given color.

      while (1) {
     
sample = 0;

//       Polling battery status bit
      for (i = 0; i < 49; i++) {
          if (STAT) {
            sample++;
          }
          delay_ms(3);
        }
 
        adc_rd = ADC1_Get_Sample(8);      // read analog value from ADC1 module channel 0
        voltage_value = (adc_rd * 5.4)/1024;          // Counts votage from read ADC value
        FloatToStr(voltage_value, txt);                 // Converts a floating point number to a string
        txt[4] = 0;                               // two decimals only

        // Delete starting message
        TFT_Set_Pen(CL_WHITE, 1);                 // Sets WHITE color and thickness parameter for drawing
        TFT_Set_Brush(1, CL_WHITE, 0, 0, 0, 0);   // Sets GREEN color and gradient which will be used to fill circles or rectangles.
        TFT_Rectangle(50, 150, 150, 200);         // Draws a rectangle on TFT

        // Display current voltage value message
        TFT_Write_Text("Current  Voltage  value (V) :", 50, 130);
        TFT_Write_Text(txt, 50, 150);



        Delay_ms(100);

       if (sample > 40){
          
          // Delete charging message
          TFT_Set_Pen(CL_WHITE, 1);
          TFT_Set_Brush(1,CL_WHITE,0, 0, 0, 0);
          TFT_Rectangle(50, 100, 200, 120);

          // Draw battery
          TFT_Set_Pen(CL_LIME, 1);                  // Sets GREEN color and thickness parameter for drawing
          TFT_Set_Brush(1, CL_LIME, 0, 0, 0, 0);    // Sets GREEN color and gradient which will be used to fill circles or rectangles.
          TFT_Rectangle(8, 6, 202, 50);             // Draws a rectangle on TFT.

          TFT_Set_Pen(CL_LIME, 1);                  // Sets GREEN color and thickness parameter for drawing
          TFT_Set_Brush(1, CL_LIME, 0, 0, 0, 0);    // Sets GREEN color and gradient which will be used to fill circles or rectangles.
          TFT_Rectangle(202, 10, 212, 46);          // Draws a rectangle on TFT

          TFT_Set_Pen(CL_LIME, 1);                  // Sets GREEN color and thickness parameter for drawing
          TFT_Set_Brush(1, CL_LIME, 0, 0, 0, 0);    // Sets GREEN color and gradient which will be used to fill circles or rectangles.
          TFT_Rectangle(212, 20, 217, 36);          // Draws a rectangle on TFT

          // Display discharging battery
          TFT_Write_Text("Discharging battery...", 50, 100);         // Writes text on the TFT at coordinates (50, 100).
          Delay_ms(2000);}
        else {
          // Delete discharging message
          TFT_Set_Pen(CL_WHITE, 1);                 // Sets WHITE color and thickness parameter for drawing
          TFT_Set_Brush(1, CL_WHITE, 0, 0, 0, 0);   // Sets WHITE color and gradient which will be used to fill circles or rectangles.
          TFT_Rectangle(50, 100, 200, 120);         // Draws a rectangle on TFT
          
          TFT_Write_Text("Charging  battery...", 50, 100);   // Writes text on the TFT at coordinates (50, 100).

          // Draw battery
          TFT_Set_Pen(CL_RED, 1);                   // Sets RED color and thickness parameter for drawing
          TFT_Set_Brush(1, CL_RED, 0, 0, 0, 0);     // Sets RED color and gradient which will be used to fill circles or rectangles.
          TFT_Rectangle(8, 6, 202, 50);             // Draws a rectangle on TFT

          TFT_Set_Pen(CL_RED, 1);                   // Sets RED color and thickness parameter for drawing
          TFT_Set_Brush(1,CL_RED, 0, 0, 0, 0);      // Sets RED color and gradient which will be used to fill circles or rectangles.
          TFT_Rectangle(202, 10, 212, 46);          // Draws a rectangle on TFT

          TFT_Set_Pen(CL_RED, 1);                   // Sets RED color and thickness parameter for drawing
          TFT_Set_Brush(1, CL_RED, 0, 0, 0, 0);     // Sets RED color and gradient which will be used to fill circles or rectangles.
          TFT_Rectangle(212, 20, 217, 36);          // Draws a rectangle on TFT

          // Draw battery charging
          for (i = 0; i < 190; i = i+10) {
            TFT_Set_Pen(CL_LIME, 1);                // Sets GREEN color and thickness parameter for drawing
            TFT_Set_Brush(1, CL_LIME, 0, 0, 0, 0);  // Sets GREEN color and gradient which will be used to fill circles or rectangles.
            TFT_Rectangle(10, 8, 20+i, 48);         // Draws a rectangle on TFT
            Delay_100ms();
          }

          for (i = 0; i < 9; i = i++) {
            TFT_Set_Pen(CL_LIME, 1);                // Sets GREEN color and thickness parameter for drawing
            TFT_Set_Brush(1, CL_LIME, 0, 0, 0, 0);  // Sets GREEN color and gradient which will be used to fill circles or rectangles.
            TFT_Rectangle(202, 12, 202+i, 44);      // Draws a rectangle on TFT
            Delay_100ms();
          }

          for (i = 0; i < 4; i++) {
            TFT_Set_Pen(CL_LIME, 1);                // Sets GREEN color and thickness parameter for drawing
            TFT_Set_Brush(1, CL_LIME, 0, 0, 0, 0);  // Sets GREEN color and gradient which will be used to fill circles or rectangles.
            TFT_Rectangle(212, 22, 212+i, 34);      // Draws a rectangle on TFT
            Delay_100ms();
          }
       }

     }
}