/*
 * Project name:
     Serial_Flash
 * Copyright:
     (c) MikroElektronika, 2010.
 * Revision History:
     20101112:
       - initial release
     20110906(JK):
       - modified for Hw. Rev. 1.10
 * Description:
      Serial flash unit. This example presents using of Serial Flash memory.
      Your TFT should blinks.
 * Test configuration:
     MCU:             PIC24FJ256GB110
                      http://ww1.microchip.com/downloads/en/DeviceDoc/39897c.pdf
     Dev.Board:       mikroMMB for PIC24
                      http://www.mikroe.com/eng/products/view/587/mikrommb-for-pic24-board/
     Oscillator:      XT-PLL 4x, 8.00000 MHz
     Modules:         ac:Serial_flash
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/eng/products/view/231/mikroc-pro-for-dspic30-33-and-pic24/
 * NOTES:
        - none.
 */

// TFT module connections
char TFT_DataPort at LATE;
char TFT_DataPort_Direction at TRISE;
sbit TFT_RST at LATC1_bit;
sbit TFT_RS at LATB15_bit;
sbit TFT_CS at LATF12_bit;
sbit TFT_RD at LATD5_bit;
sbit TFT_WR at LATD4_bit;
sbit TFT_RST_Direction at TRISC1_bit;
sbit TFT_RS_Direction at TRISB15_bit;
sbit TFT_CS_Direction at TRISF12_bit;
sbit TFT_RD_Direction at TRISD5_bit;
sbit TFT_WR_Direction at TRISD4_bit;
sbit TFT_D7 at LATE7_bit;
sbit TFT_D6 at LATE6_bit;
sbit TFT_D5 at LATE5_bit;
sbit TFT_D4 at LATE4_bit;
sbit TFT_D3 at LATE3_bit;
sbit TFT_D2 at LATE2_bit;
sbit TFT_D1 at LATE1_bit;
sbit TFT_D0 at LATE0_bit;
 
sbit CS_Serial_Flash_bit at LATC2_bit;
sbit CS_Serial_Flash_Direction_bit at TRISC2_bit;

#define _DATA_ARRAY_SIZE 16
#include <built_in.h>
#include "Serial_Flash_driver.h"

unsigned char write_array[_DATA_ARRAY_SIZE] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15};
unsigned char read_array[_DATA_ARRAY_SIZE] = {0};

void main() {
  unsigned char temp, SerialFlashID, txt[12];
  unsigned char i, to_write, success;
  unsigned long address;
  unsigned int half_address;

  AD1PCFGL = 0xFFFF;
  AD1PCFGH = 0xFFFF;

  PPS_Mapping(19, _OUTPUT, _SDO2);
  PPS_Mapping(21, _OUTPUT, _SCK2OUT);
  PPS_Mapping(26 ,_INPUT, _SDI2);

  Delay_ms(150);

  TFT_Init(320, 240);

  TFT_Fill_Screen(CL_AQUA);

  TRISG12_bit = 0;
  LATG12_bit = 1;

  //Disable other peripheral modules on the same SPI bus
  TRISG9_bit = 0;
  LATG9_bit = 1;    // Disable MMC
  TRISG12_bit = 0;
  LATG12_bit = 1;   // Disable MP3 module
  //

  SPI2_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, _SPI_PRESCALE_SEC_2, _SPI_PRESCALE_PRI_1, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_END, _SPI_CLK_IDLE_LOW, _SPI_IDLE_2_ACTIVE);

  SerialFlash_init();
  SerialFlash_WriteEnable();
  Delay_ms(500);

  TFT_Write_Text("***Serial Flash Demo***", 70, 15);

    /************************************************************************
    * Erase entire flash
    ************************************************************************/
      TFT_Write_Text("Erasing chip... ", 110, 45);
      SerialFlash_ChipErase();
      TFT_Write_Text("Done!", 110, 65);

    /************************************************************************
    * Read Flash ID
    ************************************************************************/

      SerialFlashID = SerialFlash_ReadID();
      ByteToStr(SerialFlashID, txt);
      TFT_Write_Text("Flash ID:", 110, 95);
      TFT_Write_Text(txt, 180, 95);
      Delay_ms(500);

    /************************************************************************
    * Write and read back a single byte
    ************************************************************************/


      temp = 221;
      TFT_Write_Text("Writting:", 110, 125);
      ByteToStr(temp, txt);
      TFT_Write_Text(txt, 180, 125);
      SerialFlash_WriteByte(temp, 0x123456);

      TFT_Write_Text("Reading: ", 110, 145);
      temp = SerialFlash_ReadByte(0x123456);
      ByteToStr(temp, txt);
      TFT_Write_Text(txt, 180, 145);
      Delay_ms(500);
    /************************************************************************
    * Write the repetitive pattern of data to fill the first 4K of memory
    * and then read the entire flash again and check the data correctness
    ************************************************************************/
      TFT_Write_Text("Writing array...", 110, 175);
      SerialFlash_WriteArray(0x0000, &write_array, _DATA_ARRAY_SIZE);
      Delay_ms(500);
      TFT_Write_Text("Reading array...", 110, 195);
      SerialFlash_ReadArray(0x0000, &read_array, _DATA_ARRAY_SIZE);
      Delay_ms(500);
      success = 1;
      for (i = 0; i<_DATA_ARRAY_SIZE; i++){
        if (read_array[i] != write_array[i]){
          success = 0;
          break;
        }
      }
      if (success)
      TFT_Write_Text("Success - Full match!", 110, 215);
      else
      TFT_Write_Text("Failed", 110, 215);
}