/* Project name:
     UART
 * Copyright:
     (c) Mikroelektronika, 2011.
 * Revision History:
     20110625(FJ):
       - initial release;
     20110906(JK):
       - modified for Hw. Rev. 1.10
 * Description:
     This simple example demonstrates usage of mikroC's UARTx libraries, through
     a 'loopback' interface. The data being sent to PIC32 through UART
     and sent back.
 * Test configuration:
     MCU:             PIC24FJ256GB110
                      http://ww1.microchip.com/downloads/en/DeviceDoc/39897c.pdf
     Dev.Board:       PIC32MX7 Multi Media Board
                      http://www.mikroe.com/eng/products/view/573/multimedia-board-for-pic32mx7/
     Oscillator:      HS-PLL, 32.000 MHz Crystal
     Ext. Modules:    None.
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/eng/products/view/231/mikroc-pro-for-dspic30-33-and-pic24/
 * NOTES:
     - None.
 */

char uart_rd;

void main() {
  AD1PCFG = 0xFFFF;               // Configure AN pins as digital I/O

  PPS_Mapping_NoLock(22, _INPUT, _U1RX);   // Sets pin 15 to be Input, and maps RX2/DT2 Input to it
  PPS_Mapping_NoLock(24, _OUTPUT, _U1TX);   // Sets pin 5 to be Output, and maps EUSART2 Asynchronous Transmit/Synchronous Clock Output to it


  UART1_Init(56000);              // Initialize UART module at 56000 bps
  Delay_ms(100);                  // Wait for UART module to stabilize

  UART1_Write_Text("Start");
  UART1_Write(13);
  UART1_Write(10);

  while (1) {                     // Endless loop
    if (UART1_Data_Ready()) {     // If data is received
      uart_rd = UART1_Read();     // read the received data
      UART1_Write(uart_rd);       // and send data via UART
    }
  }
}